/**************************************************************************//**
 * @file     main.c
 * @brief
 *           Phase shift waveform of PWM1 Channel 0, 2, 4 made by Synchronous function.
 *
 * @note
 * Copyright (C) 2018 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NuMicro.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Macro, type and constant definitions                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
#define PLL_CLOCK       144000000


/*---------------------------------------------------------------------------------------------------------*/
/* Global variables                                                                                        */
/*---------------------------------------------------------------------------------------------------------*/

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Enable HIRC clock (Internal RC 22.1184MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for HIRC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Select HCLK clock source as HIRC and and HCLK clock divider as 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Enable HXT clock (external XTAL 12MHz) >*/
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for HXT clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);

    /* Waiting for PLL clock ready */
    CLK_WaitClockReady(CLK_STATUS_PLLSTB_Msk);

    /* Enable PWM1 module clock */
    CLK_EnableModuleClock(EPWM1_MODULE);

    /*---------------------------------------------------------------------------------------------------------*/
    /* PWM clock frequency configuration                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Select HCLK clock source as PLL and and HCLK clock divider as 2 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL, CLK_CLKDIV0_HCLK(2));

    /* PWM clock frequency can be set equal or double to HCLK by choosing case 1 or case 2 */
    /* case 1.PWM clock frequency is set equal to HCLK: select PWM module clock source as PCLK */
    CLK_SetModuleClock(EPWM1_MODULE, CLK_CLKSEL2_EPWM1SEL_PCLK1, NULL);

    /* case 2.PWM clock frequency is set double to HCLK: select PWM module clock source as PLL */
    //CLK_SetModuleClock(PWM1_MODULE, CLK_CLKSEL2_PWM1SEL_PLL, NULL);
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source as HXT and UART module clock divider as 1 */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HXT, CLK_CLKDIV0_UART0(1));

    /* Reset PWM1 module */
    SYS_ResetModule(EPWM1_RST);

    /* Update System Core Clock */
    SystemCoreClockUpdate();


    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set PB multi-function pins for UART0 RXD and TXD */
    SYS->GPB_MFPH &= ~(SYS_GPB_MFPH_PB12MFP_Msk | SYS_GPB_MFPH_PB13MFP_Msk);
    SYS->GPB_MFPH |= (SYS_GPB_MFPH_PB12MFP_UART0_RXD | SYS_GPB_MFPH_PB13MFP_UART0_TXD);
    
    /* Set PC&PB multi-function pins for PWM1 Channel 0,2,4 */
    SYS->GPC_MFPH = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC12MFP_Msk));
	SYS->GPC_MFPH |= SYS_GPC_MFPH_PC12MFP_EPWM1_CH0;
    SYS->GPC_MFPH = (SYS->GPC_MFPH & (~SYS_GPC_MFPH_PC10MFP_Msk));
	SYS->GPC_MFPH |= SYS_GPC_MFPH_PC10MFP_EPWM1_CH2;
    SYS->GPB_MFPL = (SYS->GPC_MFPH & (~SYS_GPB_MFPL_PB7MFP_Msk));
    SYS->GPB_MFPL |= SYS_GPB_MFPL_PB7MFP_EPWM1_CH4 ;

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART module */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 baud rate */
    UART_Open(UART0, 115200);
}


/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{
    /* Init System, IP clock and multi-function I/O
       In the end of SYS_Init() will issue SYS_LockReg()
       to lock protected register. If user want to write
       protected register, please issue SYS_UnlockReg()
       to unlock protected register if necessary */

    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, IP clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();

    /* Init UART to 115200-8n1 for print message */
    UART0_Init();

    printf("\n\nCPU @ %dHz(PLL@ %dHz)\n", SystemCoreClock, PllClock);
    printf("PWM1 clock is from %s\n", (CLK->CLKSEL2 & CLK_CLKSEL2_EPWM1SEL_Msk) ? "CPU" : "PLL");
    printf("+------------------------------------------------------------------------+\n");
    printf("|                       PWM Synchronous Sample Code                      |\n");
    printf("|                                                                        |\n");
    printf("+------------------------------------------------------------------------+\n");
    printf("  This sample code will use PWM1 channel 0,2,4 to do phase shift waveform\n");
    printf("  I/O configuration:\n");
    printf("    waveform output pin: PWM1 channel 0(PC.12)\n");
    printf("                         PWM1 channel 2(PC.10)\n");
    printf("                         PWM1 channel 4(PB.7)\n");

    /* Assume PWM output frequency is 250Hz and duty ratio is (200/3)%, user can calculate PWM settings by follows.
       duty ratio = (CMR+1)/(CNR+1)
       cycle time = CNR+1
       High level = CMR+1
       PWM clock source frequency = PLL = 72000000Hz
       (CNR+1) = PWM clock source frequency/prescaler/PWM output frequency
               = 72000000/5/250 = 57600
       (Note: CNR is 16 bits, so if calculated value is larger than 65536, user should increase prescale value.)
       CNR = 57599
       duty ratio = (200/3)% ==> (CMR+1)/(CNR+1) = (200/3)%
       CMR = 38399
       Prescale value is 4 : prescaler= 5
    */

    /* Configure PWM1 generator and get the nearest frequency in edge aligned auto-reload mode */
    EPWM_ConfigOutputChannel(EPWM1, 0, 250, 200/3);
    EPWM_EnableOutput(EPWM1, EPWM_CH_0_MASK);
      
    EPWM_ConfigOutputChannel(EPWM1, 2, 250, 200/3);
    EPWM_EnableOutput(EPWM1, EPWM_CH_2_MASK);
    
    EPWM_ConfigOutputChannel(EPWM1, 4, 250, 200/3);
    EPWM_EnableOutput(EPWM1, EPWM_CH_4_MASK);

    /* Configure synchronization phase of selected channel
       Start Phase = cycle time / 3 = 19200;
       Second Phase = Start Phase + (1/2) * Start Phase = 28800; */
    EPWM_ConfigSyncPhase(EPWM1, 0, EPWM_SYNC_OUT_FROM_SYNCIN_SWSYNC, 1, 19200);
    EPWM_ConfigSyncPhase(EPWM1, 2, EPWM_SYNC_OUT_FROM_SYNCIN_SWSYNC, 1, 38400);
    EPWM_ConfigSyncPhase(EPWM1, 4, EPWM_SYNC_OUT_FROM_SYNCIN_SWSYNC, 1, 67200);
    
    /* Enable SYNC phase of selected channel */
    EPWM_EnableSyncPhase(EPWM1, 0);
    EPWM_EnableSyncPhase(EPWM1, 2);
    EPWM_EnableSyncPhase(EPWM1, 4);
    
    /* Enable PWM1 complementary mode */
    EPWM_ENABLE_COMPLEMENTARY_MODE(EPWM1);

    /* Start PWM1 module */
    EPWM_Start(EPWM1, EPWM_CH_0_MASK);
    EPWM_Start(EPWM1, EPWM_CH_2_MASK);
    EPWM_Start(EPWM1, EPWM_CH_4_MASK);
    
    /* PWM1 Software Control Synchronization Register 
       Set this register to 1 to do synchronous phase shift */
    EPWM1->SWSYNC=0x7;
    

    while(1);

}
